#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Dec 29 12:27:08 2022

@author: oursblanc

Python program to retrieve Magic card scans in English from Scryfall

How the program works
Step 1 : Check what we have in local drive
Step 2 : get the list of cards in english (the English names being authoritative)
Step 3 : for each card, try to download the visual from Scryfall
Step 4 : scan the map to detect the "Localized Image not Available FR" buffer
Step 5 : remove the visuals not in english
"""

#import libraries

import os
import re
import requests
import concurrent.futures
import cv2

## STEP 0 : setup

# setup path
cur_path = os.getcwd()+"/"
trash_path = cur_path+"Trash/"
french_path = cur_path+"French/"

# create folder
if not os.path.exists("Trash"):
   os.makedirs("Trash")
   
if not os.path.exists("French"):
   os.makedirs("French")
   
# list to store files
LocalLibrary = list()
all_urls=list()
Names=list()

## STEP 1 : create the local library to cCheck what we have in local drive

## show message to user
print("Please waiting...")

#gets files in french and trash folder
l=os.listdir(french_path)
LocalLibrary=[x.split('.')[0] for x in l]

l=os.listdir(trash_path)
LocalLibrary.extend([x.split('.')[0] for x in l])

## show message to user
print("Local libary is now updated with "+str(len(LocalLibrary))+" cards.")
print("Please waiting...")


## STEP 2 : get the list of cards in english (the English names being authoritative)

# Setup
# Find last page for search :
baseURL = 'https://deckbox.org/games/mtg/cards?p='
result = requests.get(baseURL)
LastPage=re.findall(r'(?<=span>Page 1 of )(.*)(?=</span>)', result.text)
LastPage=1+int(LastPage[0])

## prepare all URL
def generate_urls():
    for i in range (1,LastPage):
        all_urls.append(baseURL+str(i))

## prepare the scrap
def scrap(url):
    result = requests.get(url)
    Names.extend(re.findall(r'(?<=blank\'>)(.*)(?=</a>)',result.text))

## generate all the url to scan
generate_urls()

## make concurrent requests to parallelize processing
with concurrent.futures.ThreadPoolExecutor(max_workers=12) as executor:
    executor.map(scrap, all_urls)
    
## show message to user
print("List of " + str(len(Names)) + " cards are loaded from deckbox.org.")
print("Please waiting...")

## Step 3 : download the image
def scan(cardname):
    # if the cardname is not in the local library so we download the img
    if cardname not in LocalLibrary:
        # with base URL (for search)
        baseURL = 'https://scryfall.com/search?q='
        # complete URL with the card name
        fullURL=baseURL+str(cardname)
        # extract the HTML code
        result = requests.get(fullURL)
        # with regex, find the french version of the page
        frenchURL=re.findall(r'(?<=href=")(.*)(?=">fr</a>)', result.text)  
        # in the french version, try to catch the scan of the card
        result = requests.get(frenchURL[0])
        # first clean (catch approx. the piece of HTML code)
        cardURL=re.findall(r'(?<=<img class=)(.*)(?=/>)', result.text)
        # second clean (catch exactly the img URL)
        cardURL=re.findall(r'(?<=src=")(.*)(?=")', cardURL[0])
        # download the card through link found before
        img_data = requests.get(cardURL[0]).content
        # save the card
        jpgFile=cardname+str(".jpg")
        with open(jpgFile, 'wb') as handler:
            handler.write(img_data)
            
        #check if the image have the stamp "Not Available in French"
        img = cv2.imread(jpgFile,0)
        img2 = img.copy()
        template = cv2.imread('STAMP.png',0)
        w, h = template.shape[::-1]

        img = img2.copy()
        method = eval("cv2.TM_CCORR_NORMED")

        # Apply template Matching
        res = cv2.matchTemplate(img,template,method)
        min_val, max_val, min_loc, max_loc = cv2.minMaxLoc(res)

        top_left = max_loc
        bottom_right = (top_left[0] + w, top_left[1] + h)

        cv2.rectangle(img,top_left, bottom_right, 255, 2)

        # show result 
        """
        plt.subplot(121),plt.imshow(res,cmap = 'gray')
        plt.title('Matching Result'), plt.xticks([]), plt.yticks([])
        plt.subplot(122),plt.imshow(img,cmap = 'gray')
        plt.title('Detected Point'), plt.xticks([]), plt.yticks([])
        plt.suptitle("cv2.TM_CCORR_NORMED")
        plt.show()
        """

        if top_left[0]<170 or top_left [0]>180 or top_left[1]<315 or top_left [1]>320:
            print(str(jpgFile)+" => Image with French founded !")
            #move file in French folder
            origin=os.getcwd()+"/"
            target=os.getcwd()+"/French/"
            os.rename(origin+jpgFile,target+jpgFile)
            

        else:
            print(str(jpgFile)+" => Image with French not available !")
            #move file in trash folder
            origin=os.getcwd()+"/"
            target=os.getcwd()+"/Trash/"
            os.rename(origin+jpgFile,target+jpgFile)
        
        
## make concurrent requests to parallelize processing
with concurrent.futures.ThreadPoolExecutor(max_workers=12) as executor:
    executor.map(scan, Names)    